import React, { useEffect, useRef, useState } from "react";
import { StyleSheet, View, Image, Text, StatusBar } from "react-native";
import {
  SCREEN_HEIGHT,
  SCREEN_WIDTH,
} from "react-native-digitcamera/src/common/Constants";
import { translate } from "react-native-digitcamera/src/common/localize/i18n";
import Orientation from "../../../react-native-orientation";
import Geolocation from "@react-native-community/geolocation";
import { TouchableOpacity } from "react-native-gesture-handler";
import RNFetchBlob from "rn-fetch-blob";
import { SELFSURVEYSERVICE_DOCUMENT_URL } from "../Api/ApiUrls";
import ViewShot, { captureRef } from "react-native-view-shot";
import VideoPlayer from "react-native-video-player";
import { poseDetection } from "react-native-digitcamera-pi/src/common/Services/poseDetection";
import {
  CHASSIS_NUMBER,
  ACTIVITY_TYPE_MOTOR_2W_PRE_INSPECTION_SELF_SURVEY,
  ACTIVITY_TYPE_MOTOR_4W_OD_NB_PRE_INSPECTION_SELF_SURVEY,
  ACTIVITY_TYPE_MOTOR_PRE_INSPECTION_SELF_SURVEY,
} from "../common/Constants";
import { chassiesPoseDetetction } from "../common/Services/chassisPoseDetection";

//Global variables
let isBlur = true,
  isDark = true;
const PreviewImage = ({ navigation, route }) => {
  const imageref = useRef();
  const [currentLongitude, SetCurrentLongitude] = useState("Unknown");
  const [currentLatitude, SetCurrentLatitude] = useState("Unknown");
  const [dateAndTime, SetDateAndTime] = useState("");
  const [imageName, SetImageName] = useState("");
  const [finalImagePath, SetFinalImagePath] = useState("");

  const {
    ImageData,
    ext,
    DocumentImageName,
    activityId,
    activityTypeId,
    webUserId,
  } = route.params;

  const retake = require("../common/Image/retake.png");
  const proceed = require("../common/Image/proceed.png");

  useEffect(() => {
    getTime();
    getLocation();
    Orientation.lockToLandscapeLeft();
    // callLocation()
  }, []);
  const getLocation = () => {
    Geolocation.getCurrentPosition((info) => {
      SetCurrentLongitude(info.coords.longitude);
      SetCurrentLatitude(info.coords.latitude);
    });
  };
  const getTime = () => {
    const date = new Date().getDate();
    const month = new Date().getMonth() + 1;
    const year = new Date().getFullYear();

    //Time calulation for the display on preciew Screen
    const hour = new Date().getHours();
    const minute = new Date().getMinutes();
    const seconds = new Date().getSeconds();
    SetDateAndTime(
      date +
        "/" +
        month +
        "/" +
        year +
        " - " +
        hour +
        ":" +
        minute +
        ":" +
        seconds
    );
  };

  //CALL CHASSIS POSE DETECTION
  const callChassisPoseDetection = async () => {
    var formdata = new FormData();
    console.log("finalImagePath", finalImagePath);
    formdata.append("file", {
      uri: finalImagePath,
      name: activityId + "_" + DocumentImageName + ext,
      fileName: activityId + "_" + DocumentImageName,
      type: "image/jpeg",
    });
    formdata.append("activityId", activityId);

    const ChassisPoseDetectionData = await chassiesPoseDetetction({
      body: formdata,
    });

    console.log(
      " >>>>>>>>>>>>>>>>>>>>>>>>>  >>>  ",
      JSON.stringify(ChassisPoseDetectionData, null, "  ")
    );
  };
  // POSE DETECTION API CALL

  const callPoseDetection = async () => {
    var formdata = new FormData();
    console.log("finalImagePath", finalImagePath);
    formdata.append("file", {
      uri: finalImagePath,
      name: activityId + "_" + DocumentImageName + ext,
      fileName: activityId + "_" + DocumentImageName,
      type: "image/jpeg",
    }),
      formdata.append("policyNumber", "MH12EX1234"),
      formdata.append("activityTypeId", "5"),
      formdata.append("checkQuality", true),
      formdata.append("source", "DIGITEDGE");

    const poseDetectionData = await poseDetection({
      body: formdata,
    });
    isBlur = poseDetectionData.image_pose_array[0].is_blurred;
    isDark = poseDetectionData.image_pose_array[0].is_dark;

    console.log("isBlur", isBlur);
    console.log("isDark", isDark);

    if (!isBlur && !isDark) {
      //if image is good then excute this block
      // upload image
      uploadImage();
      //IN both case dispaly next images
      navigation.navigate("PI");
    } else {
      // GO FOR RE-TRY

      navigation.navigate("PI");
    }
  };
  //upload images
  const uploadImage = () => {
    const documentUploadData = {
      activityId: activityId,
      documentSource: "DigitEdge",
      geoLocation: {
        deviceIMEI: "",
        deviceName: "",
        deviceTime: "",
        documentSource: "DigitEdge",
        ipAddress: "",
        isBlocked: false,
        latitude: "",
        longitude: "",
        macAddress: "",
        operatingSystem: "",
        timezone: "Asia/Kolkata",
      },
      isLast: false,
      isLastData: false,
      isLastDocument: false,
      isSurveyor: false,
      manualOdometerReading: "",
      source: "DIGITEDGE",
    };
    console.log(" FILE NAME   ", activityId + "_" + DocumentImageName + ext);
    RNFetchBlob.fetch(
      "POST",
      SELFSURVEYSERVICE_DOCUMENT_URL,
      {
        "Content-Type": "multipart/form-data",
      },
      [
        // element with property `filename` will be transformed into `file` in form data
        //file name is Original file name
        {
          name: "file",
          filename: activityId + "_" + DocumentImageName + ext,
          type: "image/jpg",
          data: finalImagePath,
        },
        { name: "documentData", data: JSON.stringify(documentUploadData) },
      ]
    )
      .uploadProgress({ interval: 250 }, (written, total) => {
        console.log("Written", written);
        console.log("total", total);
        console.log("uploaded", written / total);
        // this.setState({ documentUploadPercentage: (written / total) })
        // this.props.addUploadImageDatasource(
        //     {
        //         imagePath: 'file://' + this.state.FilePath + this.state.imageName,
        //         stepTitle: this.state.stepTitle,
        //         imageSize: this.state.fileSize,
        //         upLoadStatus: this.state.documentUploadPercentage,
        //     }
        // )
        // if (this.state.count == this.state.instructionData.length - 1) {
        //     // if (this.state.count == 1) {
        //     this.props.navigation.navigate("DocumentUpload", { upLoadIamgeDataSource: this.state.upLoadIamgeDataSource })
        // }
      })
      // listen to download progress event, every 10%
      .progress({ count: 10 }, (received, total) => {
        console.log("progress", received / total);
        // let isExisist = this.state.upLoadIamgeDataSource.find(this.state.upLoadIamgeDataSource.stepTitle == this.state.instructionData[this.state.count - 1].stepTitle)
        // if (this.state.count == this.state.instructionData.length) {
        //     // if (this.state.count == 1) {
        //     this.props.navigation.navigate("DocumentUpload", { upLoadIamgeDataSource: this.state.upLoadIamgeDataSource })
        // }
      })

      .then((response) => {})
      .catch((err) => {
        console.log("******************** err: " + JSON.stringify(err));
      });
  };
  // create image with info
  const createImageWithName = async () => {
    if (DocumentImageName == "Take a 360° video") {
      console.log("IMAGE DATA", ImageData);
      SetFinalImagePath(ImageData);
      uploadImage();
    } else {
      try {
        await captureRef(imageref, {
          format: "jpg",
          quality: 0.8,
        }).then(
          (uri) => {
            SetFinalImagePath(uri);
            if (DocumentImageName === CHASSIS_NUMBER) {
              // call API for chassies number pose detection
              callChassisPoseDetection();
            } else if (
              activityTypeId ===
              ACTIVITY_TYPE_MOTOR_2W_PRE_INSPECTION_SELF_SURVEY
            ) {
              // call APi for pose detection for 2W
            } else if (
              activityId === ACTIVITY_TYPE_MOTOR_PRE_INSPECTION_SELF_SURVEY ||
              activityId ===
                ACTIVITY_TYPE_MOTOR_4W_OD_NB_PRE_INSPECTION_SELF_SURVEY
            ) {
              // call API for 4W pose detection
              callPoseDetection();
            }
          },

          (error) => console.error("Oops, snapshot failed", error)
        );
      } catch (err) {}
    }
  };
  return (
    <View style={styles.main}>
      <StatusBar hidden={true} />
      {/* Image View */}
      <View style={styles.imageView}>
        {/* Image */}
        {DocumentImageName == "Take a 360° video" ? (
          <VideoPlayer
            video={{ uri: ImageData }}
            videoWidth={SCREEN_HEIGHT}
            videoHeight={SCREEN_WIDTH}
            pauseOnPress={true}
            seekBarWidth={5}
          ></VideoPlayer>
        ) : (
          <ViewShot ref={imageref}>
            <View>
              <Image
                source={{ uri: ImageData.uri }}
                style={{
                  width: SCREEN_HEIGHT,
                  height: SCREEN_WIDTH,
                  resizeMode: "contain",
                }}
              />
              {/* info Strip */}
              <View style={styles.infoTextView}>
                <Text style={styles.infoText}>
                  {translate("Lat:") +
                    " " +
                    currentLatitude +
                    "  " +
                    translate("Lon:") +
                    "  " +
                    currentLongitude +
                    "  " +
                    "    |    " +
                    "Digit App" +
                    "    |    " +
                    dateAndTime}
                </Text>
              </View>
            </View>
          </ViewShot>
        )}
      </View>
      {/* Button View */}
      <View style={styles.buttonView}>
        {/* Retake View */}
        <View style={styles.retakeView}>
          <TouchableOpacity
            onPress={() => {
              navigation.navigate("Camera");
            }}
          >
            <Image
              source={retake}
              style={{ width: 18, height: 18, alignSelf: "center" }}
            />
            <Text style={styles.retakeText}>{translate("Retake")}</Text>
          </TouchableOpacity>
        </View>
        {/* Proceed View */}
        <View style={styles.proceedView}>
          <TouchableOpacity
            onPress={() => {
              createImageWithName();
            }}
          >
            <Image
              source={proceed}
              style={{ width: 49, height: 49, alignSelf: "center" }}
            />
            <Text style={styles.retakeText}>{translate("Proceed")}</Text>
          </TouchableOpacity>
        </View>
        {/* Blank View */}
        <View style={styles.blankView}></View>
      </View>
    </View>
  );
};
export default PreviewImage;

const styles = StyleSheet.create({
  main: {
    flex: 1,
    flexDirection: "row",
  },
  buttonView: {
    flex: 0.1,
    backgroundColor: "#000000",
  },
  imageView: {
    flex: 0.9,
  },
  retakeView: {
    flex: 0.333333,
    justifyContent: "center",
  },
  proceedView: {
    flex: 0.333333,
    justifyContent: "center",
  },
  blankView: {
    flex: 0.3333333,
  },
  retakeText: {
    fontFamily: "Muli-Regular",
    fontSize: 11,
    alignSelf: "center",
    marginTop: 5,
    color: "#FFFFFF",
  },
  infoTextView: {
    // flex:0.1,
    position: "absolute",
    width: SCREEN_HEIGHT,
    height: 48,
    backgroundColor: "#000000",
    opacity: 0.58,
    justifyContent: "center",
  },
  infoText: {
    fontFamily: "Muli-SemiBold",
    fontSize: 14,
    color: "#FFFFFF",
    marginLeft: 24,
    marginRight: 24,
  },
});
